@@ -6,8 +6,8 @@
 
 read.shape <- function(filen, dbf.data=TRUE, verbose=TRUE, repair=FALSE) {
   filen <- path.expand(filen)
-  .Deprecated("", package="maptools", msg="use readShapeSpatial:\nobjects other than Spatial objects defined in the sp package are deprecated")
-  if (length(grep("\\.shp$", tolower(filen))) == 0)
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
+  if (length(grep("\\.shp$", tolower(filen))) == 0L)
     filen <- paste(filen, "shp", sep=".")
   shinfo <- getinfo.shape(filen)
   if (dbf.data) {
@@ -20,9 +20,9 @@ read.shape <- function(filen, dbf.data=TRUE, verbose=TRUE, repair=FALSE) {
     lsbn <- length(sbn)
     if (lsbn > 1 && tolower(sbn[lsbn]) == "shp") sbn[lsbn] <- "dbf"
     filen1 <- paste(sbn, collapse=".")
-    if (length(grep("\\.dbf$", filen1)) == 0)
+    if (length(grep("\\.dbf$", filen1)) == 0L)
         filen1 <- paste(filen1, "dbf", sep=".")
-    if (length(dn) > 0) {
+    if (length(dn) > 0L) {
         filen1 <- paste(dn, filen1, sep=.Platform$file.sep)
     }
     df <- read.dbf(filen1)
@@ -56,12 +56,14 @@ read.shape <- function(filen, dbf.data=TRUE, verbose=TRUE, repair=FALSE) {
 }
 
 getinfo.shape <- function(filen) {
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
   shapehead <-.Call("Rshapeinfo1", as.character(path.expand(filen)), PACKAGE="maptools")
   class(shapehead) <- "shapehead"
   shapehead
 }
 
 print.shapehead <- function(x, ...) {
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
     types <- c("Point", NA, "PolyLine", NA, "Polygon", NA, NA, "MultiPoint", NA, NA, "PointZ", NA, "PolyLineZ", NA, "PolygonZ", NA, NA, "MultiPointZ", NA, NA, "PointM", NA, "PolyLineM", NA, "PolygonM", NA, NA, "MultiPointM", NA, NA, "MultiPatch")
     cat("Shapefile type: ", types[x[[2]]], ", (", x[[2]], "), # of Shapes: ", 
       x[[3]], "\n", sep="")
@@ -71,7 +73,7 @@ print.shapehead <- function(x, ...) {
 #write.pointShape <- function(object, file, coordinates, factor2char=TRUE, 
 write.pointShape <- function(coordinates, df, file, factor2char=TRUE, 
   strictFilename=FALSE, max_nchar=254) {
-  .Deprecated("", package="maptools", msg="use writeSpatialShape:\nobjects other than Spatial objects defined in the sp package are deprecated")
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
   file <- path.expand(file)
   dirnm <- dirname(file)
   bnm0 <- basename(file)
@@ -91,7 +93,8 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
     stop("different number of rows in coordinates and data frame")
 #  library(foreign)
   write.dbf(df, paste(file, ".dbf", sep=""), factor2char=factor2char, max_nchar=max_nchar)
-  res <- .Call("shpwritepoint", as.character(file), as.double(coordinates),
+  storage.mode(coordinates) <- "double"
+  res <- .Call("shpwritepoint", as.character(file), coordinates,
     as.integer(ncolcrds), PACKAGE="maptools")
   invisible(res)
 }
@@ -99,7 +102,7 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
 .isValidPolylist <- function(polylist, verbose=FALSE) {
   if (!inherits(polylist, "polylist")) stop("not a polylist object")
   res <- TRUE
-  if (length(polylist) < 1) {
+  if (length(polylist) < 1L) {
     if (verbose) cat("zero length polylist\n")
     res <- FALSE
   }
@@ -163,13 +166,13 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
 
 .makePolylistValid <- function(polylist) {
   if (!inherits(polylist, "polylist")) stop("not a polylist object")
-  if (length(polylist) < 1) stop("zero length polylist")
+  if (length(polylist) < 1L) stop("zero length polylist")
   n <- length(polylist)
   if (is.null(attr(polylist, "nDims")) || 
     !is.integer(attr(polylist, "nDims")) || 
     (attr(polylist, "nDims") < 2 || attr(polylist, "nDims") > 3)) {
     nD <- unique(sapply(polylist, function(x) dim(x)[2]))
-    if (length(nD) > 1) stop("multiple dimension polylist components")
+    if (length(nD) > 1L) stop("multiple dimension polylist components")
     nD <- as.integer(nD)
     attr(polylist, "nDims") <- nD
   }
@@ -186,7 +189,11 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
       if (any(is.na(c(polylist[[i]])))) {
 	xy <- polylist[[i]]
         NAs <- unclass(attr(na.omit(xy), "na.action"))
-	nParts <- length(NAs) + 1
+        if (NAs[length(NAs)] == nrow(xy)) {
+          NAs <- NAs[-length(NAs)]
+          xy <- xy[1:(nrow(xy)-1),]
+        }
+	nParts <- length(NAs) + 1L
 	from <- integer(nParts)
 	to <- integer(nParts)
 	from[1] <- 1
@@ -228,7 +235,7 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
       if (any(is.na(c(polylist[[i]])))) {
 	xy <- polylist[[i]]
         NAs <- unclass(attr(na.omit(xy), "na.action"))
-	nParts <- length(NAs) + 1
+	nParts <- length(NAs) + 1L
 	from <- integer(nParts)
 	to <- integer(nParts)
 	from[1] <- 1
@@ -273,7 +280,7 @@ write.pointShape <- function(coordinates, df, file, factor2char=TRUE,
 
 write.polylistShape <- function(polylist, df, file, factor2char=TRUE, 
   strictFilename=FALSE, force=TRUE, max_nchar=254) {
-  .Deprecated("", package="maptools", msg="use writeSpatialShape:\nobjects other than Spatial objects defined in the sp package are deprecated")
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
   file <- path.expand(file)
   dirnm <- dirname(file)
   bnm0 <- basename(file)
@@ -285,7 +292,7 @@ write.polylistShape <- function(polylist, df, file, factor2char=TRUE,
   if (strictFilename && nchar(basename(file)) > 8) 
     stop("shapefile names must conform to the 8.3 format")
   if (!inherits(polylist, "polylist")) stop("not a polylist object")
-  if (length(polylist) < 1) stop("zero length polylist")
+  if (length(polylist) < 1L) stop("zero length polylist")
   if (nrow(df) != length(polylist))
     stop("different number of rows in polylist and data frame")
   if (!.isValidPolylist(polylist)) {
@@ -302,7 +309,7 @@ write.polylistShape <- function(polylist, df, file, factor2char=TRUE,
 
 write.linelistShape <- function(linelist, df, file, factor2char=TRUE, 
   strictFilename=FALSE, max_nchar=254) {
-  .Deprecated("", package="maptools", msg="use writeSpatialShape:\nobjects other than Spatial objects defined in the sp package are deprecated")
+  .Deprecated("", package="maptools", msg="shapelib support is provided by GDAL through the sf and terra paackages among others")
   file <- path.expand(file)
   dirnm <- dirname(file)
   bnm0 <- basename(file)
@@ -313,7 +320,7 @@ write.linelistShape <- function(linelist, df, file, factor2char=TRUE,
   file <- paste(dirnm, bnm, sep=.Platform$file.sep)
   if (strictFilename && nchar(basename(file)) > 8) 
     stop("shapefile names must conform to the 8.3 format")
-  if (length(linelist) < 1) stop("zero length linelist")
+  if (length(linelist) < 1L) stop("zero length linelist")
   if (nrow(df) != length(linelist))
     stop("different number of rows in linelist and data frame")
   if (!any(sapply(linelist, function(x) is.integer(attr(x, "nParts"))))) {
@@ -351,3 +358,74 @@ write.linelistShape <- function(linelist, df, file, factor2char=TRUE,
   invisible(res)
 }
 
+readMAP2polylist = function(filename){
+  zz=file(filename,"rb")
+  #
+  # header of .map
+  #
+  versao = readBin(zz,"integer",1,size=2)  # 100 = versao 1.00
+  #Bounding Box
+  Leste = readBin(zz,"numeric",1,size=4)
+  Norte = readBin(zz,"numeric",1,size=4)
+  Oeste = readBin(zz,"numeric",1,size=4)
+  Sul   = readBin(zz,"numeric",1,size=4)
+
+  geocodigo = ""
+  nome = ""
+  xleg = 0
+  yleg = 0
+  sede = FALSE
+  poli = list()
+  i = 0
+
+  #
+  # repeat of each object in file
+  #
+  repeat{  
+    tipoobj = readBin(zz,"integer",1,size=1) # 0=Poligono, 1=PoligonoComSede, 2=Linha, 3=Ponto
+
+    if (length(tipoobj) == 0) break
+    i = i + 1
+
+    Len = readBin(zz,"integer",1,size=1)  # length byte da string Pascal
+    geocodigo[i] = rawToChar(readBin(zz, "raw", 10, size = 1))#readChar(zz,10)
+    Len = readBin(zz,"integer",1,size=1)  # length byte da string Pascal
+    inp <- readBin(zz, "raw", 25, size = 1)
+    inp[inp > 127] <- charToRaw(" ")
+    nome[i] = substr(rawToChar(inp), 1, Len)
+    xleg[i] = readBin(zz,"numeric",1,size=4)
+    yleg[i] = readBin(zz,"numeric",1,size=4)
+    numpontos = readBin(zz,"integer",1,size=2)
+
+    sede = sede || (tipoobj = 1)
+
+    x=0
+    y=0   
+    for (j in 1:numpontos){
+      x[j] = readBin(zz,"numeric",1,size=4)
+      y[j] = readBin(zz,"numeric",1,size=4)
+    }
+
+
+    # separate polygons
+    xInic = x[1]
+    yInic = y[1]  
+    for (j in 2:numpontos){
+      if (x[j] == xInic & y[j] == yInic) {x[j]=NA; y[j] = NA}
+    }
+
+    poli[[i]] = c(x,y)
+    dim(poli[[i]]) = c(numpontos,2)
+  }
+
+  class(poli) = "polylist"
+  attr(poli,"region.id") = geocodigo
+  attr(poli,"region.name") = nome
+  attr(poli,"centroid") = list(x=xleg,y=yleg)
+  attr(poli,"sede") = sede
+  attr(poli,"maplim") = list(x=c(Oeste,Leste),y=c(Sul,Norte))
+
+  close(zz)
+  return(poli)
+}
+
